/*
    __HAL_STM32.c

-----------------------------------------------------------------------------

  This file is part of mikroSDK.

  Copyright (c) 2019, MikroElektonika - http://www.mikroe.com

  All rights reserved.

----------------------------------------------------------------------------- */

#include "stdint.h"

#ifdef __MIKROC_PRO_FOR_ARM__
#ifdef __STM32__
#ifdef __HAL_SPI__

typedef void          ( *hal_spi_write_fp_t )(unsigned int);
typedef unsigned int  ( *hal_spi_read_fp_t )(unsigned int);

typedef struct
{
    hal_spi_write_fp_t       spi_write;
    hal_spi_read_fp_t        spi_read;
  
}hal_struct_spi_t;

static hal_spi_write_fp_t        fp_spi_write;                          
static hal_spi_read_fp_t         fp_spi_read;                           

static void hal_spi_map(hal_p_t spi_obj)
{
    hal_obj_spi_t tmp = (hal_obj_spi_t)spi_obj;

    fp_spi_write = tmp->spi_write;
    fp_spi_read  = tmp->spi_read;
}

static void hal_spi_write(uint8_t *p_buf, uint16_t len)
{
    uint8_t *ptr = p_buf;
    while( len-- )
        fp_spi_write( *( ptr++ ) );
}

static void hal_spi_read(uint8_t *p_buf, uint16_t len)
{
    uint8_t *ptr = p_buf;
    while( len-- )
        *( ptr++ ) = fp_spi_read( 0x00 );
}

static void hal_spi_transfer(uint8_t *p_in, uint8_t *p_out, uint16_t len)
{
    uint8_t *in = p_in;
    uint8_t *out = p_out;
    while( len-- )
        *( out++ ) = fp_spi_read( *( in++ ) );
}

#endif
#ifdef __HAL_I2C__

typedef unsigned int    ( *hal_i2c_start_fp_t )();                       
typedef unsigned int    ( *hal_i2c_write_fp_t )(unsigned char, unsigned char*, unsigned long, unsigned long);
typedef void            ( *hal_i2c_read_fp_t )(unsigned char, unsigned char*, unsigned long, unsigned long);

typedef struct
{
    hal_i2c_start_fp_t       i2c_start;
    hal_i2c_write_fp_t       i2c_write;
    hal_i2c_read_fp_t        i2c_read;
  
}hal_struct_i2c_t;

static hal_i2c_start_fp_t        fp_i2c_start;                          
static hal_i2c_write_fp_t        fp_i2c_write;                          
static hal_i2c_read_fp_t         fp_i2c_read;                           

static void hal_i2c_map(hal_p_t i2c_obj)
{
    hal_obj_i2c_t tmp = (hal_obj_i2c_t)i2c_obj;

    fp_i2c_start    = tmp->i2c_start;
    fp_i2c_write    = tmp->i2c_write;
    fp_i2c_read     = tmp->i2c_read;
}

static int hal_i2c_start()
{
    int res = 0;
    res |= fp_i2c_start();
    return res;
}

static int hal_i2c_write(uint8_t slave_addr, uint8_t *p_buf, uint16_t len, uint8_t end_mode)
{
    int res = 0;
    uint8_t *ptr = p_buf;
    res |= fp_i2c_write(slave_addr, ptr, len, end_mode);
    return res;
}

static int hal_i2c_read(uint8_t slave_addr, uint8_t *p_buf, uint16_t len, uint8_t end_mode)
{
    int res = 0;
    uint8_t *ptr = p_buf;
    fp_i2c_read(slave_addr, ptr, len, end_mode);
    return res;
}

#endif
#ifdef __HAL_UART__

typedef void          ( *hal_uart_write_fp_t )(unsigned int);
typedef unsigned int  ( *hal_uart_read_fp_t )();
typedef unsigned int  ( *hal_uart_ready_fp_t )();

typedef struct
{
    hal_uart_write_fp_t      uart_write;
    hal_uart_read_fp_t       uart_read;
    hal_uart_ready_fp_t      uart_ready;
  
}hal_struct_uart_t;

static hal_uart_write_fp_t       fp_uart_write;                         
static hal_uart_read_fp_t        fp_uart_read;                          
static hal_uart_ready_fp_t       fp_uart_ready;                         

static void hal_uart_map(hal_p_t uart_obj)
{
    hal_obj_uart_t tmp = (hal_obj_uart_t)uart_obj;

    fp_uart_write = tmp->uart_write;
    fp_uart_read  = tmp->uart_read;
    fp_uart_ready = tmp->uart_ready;
}

static void hal_uart_write(uint8_t input)
{
    fp_uart_write(input);
}

static uint8_t hal_uart_read()
{
    return ( uint8_t )fp_uart_read();
}

static uint8_t hal_uart_ready()
{
    return ( uint8_t )fp_uart_ready();
}

#endif
#endif
#endif

/* -------------------------------------------------------------------------- */
/*
  __HAL_STM32.c

  Copyright (c) 2011-2019 MikroElektronika.  All right reserved.

    This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.

----------------------------------------------------------------------------- */
